"""
Use case : cantilever beam
==========================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class CantileverBeam():
    """Custom class for the cantilever beam example.
    """
    dim: int = 4  # number of inputs
    # Young's modulus E
    E: Any = ot.Beta(0.9, 2.27, 2.5e7, 5.0e7)  # in N/m^2
    E.setDescription("E")
    E.setName("Young modulus")

    # Load F
    F: Any = ot.LogNormal()  # in N
    F.setParameter(ot.LogNormalMuSigma()([30.e3, 9e3, 15.e3]))
    F.setDescription("F")
    F.setName("Load")

    # Length L
    L: Any = ot.Uniform(250., 260.)  # in cm
    L.setDescription("L")
    L.setName("Length")

    # Moment of inertia I
    I: Any = ot.Beta(2.5, 1.5, 310, 450)  # in cm^4
    I.setDescription("I")
    I.setName("Inertia")

    model: Any = ot.SymbolicFunction(['E', 'F', 'L', 'I'], ['F*L^3/(3*E*I)'])

    R: Any = ot.CorrelationMatrix(dim)
    R[2, 3] = -0.2
    copula: Any = ot.NormalCopula(ot.NormalCopula.GetCorrelationFromSpearmanCorrelation(R))
    distribution: Any = ot.ComposedDistribution([E, F, L, I], copula)

    X: Any = ot.RandomVector(distribution)
    X.setDescription(['E', 'F', 'L', 'I'])
    Y: Any = ot.CompositeRandomVector(model, X)
    Y.setDescription('Y')
    event: Any = ot.ThresholdEvent(Y, ot.Greater(), 30.0)



"""
Use case : the Chaboche mechanical model
========================================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


def g(X):
    strain, R, C, gamma = X
    stress = R + C * (1 - np.exp(-gamma * strain)) / gamma
    return [stress]


@dataclass
class ChabocheModel():
    """Custom class for the Chaboche model
    """

    dim: int = 4  # number of inputs

    Strain: Any = ot.Uniform(0.0, 0.07)
    Strain.setDescription(["Strain"])

    unknownR: float = 750.0e6
    R: Any = ot.Dirac(unknownR)
    R.setDescription(["R"])

    unknownC: float = 2750.0e6
    C: Any = ot.Dirac(unknownC)
    C.setDescription(["C"])

    unknownGamma: float = 10.0
    Gamma: Any = ot.Dirac(unknownGamma)
    Gamma.setDescription(["Gamma"])

    inputDistribution: Any = ot.ComposedDistribution([Strain, R, C, Gamma])

    model: Any = ot.PythonFunction(4, 1, g)



"""
Use case : Branin test function
===============================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class BraninModel():
    """Custom class for the Branin test function
    """
    dim: int = 2
    trueNoiseFunction: float = 0.1

    branin: Any = ot.SymbolicFunction(['x1', 'x2'],
                                      ['((x2-(5.1/(4*pi_^2))*x1^2+5*x1/pi_-6)^2+10*(1-1/8*pi_)*cos(x1)+10-54.8104)/51.9496',
                                       str(trueNoiseFunction)])
    transfo: Any = ot.SymbolicFunction(['u1', 'u2'],
                                       ['15*u1-5', '15*u2'])

    model: Any = ot.ComposedFunction(branin, transfo)

    # %%
    lowerbound = ot.Point([0.0] * dim)
    upperbound = ot.Point([1.0] * dim)

    objectiveFunction = model.getMarginal(0)

    # Minimas
    xexact1: Any = ot.Point([0.123895, 0.818329])
    xexact2: Any = ot.Point([0.542773, 0.151666])
    xexact3: Any = ot.Point([0.961652, 0.165000])
    xexact: Any = ot.Sample([xexact1, xexact2, xexact3])



"""
Use case : axial stressed beam
==============================
"""
from __future__ import print_function
import openturns as ot
import openturns.viewer as viewer
from matplotlib import pylab as plt
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


@dataclass
class AxialStressedBeam():
    """Custom class for the beam example.
    """
    D: float = 0.02
    # Random variable : R
    muR: float = 3.0e6
    sigmaR: float = 3.0e5
    # Random variable : F
    muF: float = 750.
    sigmaF: float = 50.
    # create the limit state function model
    model: Any = ot.SymbolicFunction(['R', 'F'], ['R-F/(pi_/10000.0)'])
    # Create the joint distribution of the parameters.
    distribution_R: Any = ot.LogNormalMuSigma(muR, sigmaR, 0.0).getDistribution()
    distribution_F: Any = ot.Normal(muF, sigmaF)
    distribution: Any = ot.ComposedDistribution([distribution_R, distribution_F])

    # Create the event whose probability we want to estimate.
    vect: Any = ot.RandomVector(distribution)
    G: Any = ot.CompositeRandomVector(model, vect)
    event: Any = ot.ThresholdEvent(G, ot.Less(), 0.0)



"""
Use case : viscous free fall
============================
"""
from __future__ import print_function
import openturns as ot
from typing import Any, List
import numpy as np
from dataclasses import dataclass
from dataclasses import field


def AltiFunc(X):
    g = 9.81
    z0 = X[0]
    v0 = X[1]
    m = X[2]
    c = X[3]
    tau = m / c
    vinf = - m * g / c
    t = np.array(vertices)
    z = z0 + vinf * t + tau * (v0 - vinf) * (1 - np.exp(- t / tau))
    z = np.maximum(z, 0.)
    return [[zeta[0]] for zeta in z]


@dataclass
class ViscousFreeFall():
    """Custom class for the viscous free fall
    """
    dim: int = 4  # number of inputs

    tmin: float = 0.0  # Minimum time
    tmax: float = 12.0  # Maximum time
    gridsize: int = 100  # Number of time steps
    mesh: Any = ot.IntervalMesher([gridsize-1]).build(ot.Interval(tmin, tmax))
    vertices: Any = mesh.getVertices()
    distZ0: Any = ot.Uniform(100.0, 150.0)
    distV0: Any = ot.Normal(55.0, 10.0)
    distM: Any = ot.Normal(80.0, 8.0)
    distC: Any = ot.Uniform(0.0, 30.0)
    distribution: Any = ot.ComposedDistribution([distZ0, distV0, distM, distC])

    outputDimension: int = 1
    alti: Any = ot.PythonPointToFieldFunction(dim, mesh, outputDimension, AltiFunc)
